package com.muyu.pay.wechat.service;

import com.muyu.pay.wechat.config.WechatPayConfig;
import com.muyu.pay.wechat.domain.model.WeChatSignModel;
import com.muyu.utils.IdUtils;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import okhttp3.HttpUrl;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.*;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

import static com.muyu.pay.wechat.constant.WechatPayConstant.ALGORITHM_TYPE;

/**
 * @author DongZl
 * @description: 签名工具类
 * @Date 2023-8-21 上午 11:23
 */
@Log4j2
@Service
@AllArgsConstructor
public class WeChatSignService {
    /**
     * 微信配置
     */
    private final WechatPayConfig wechatPayConfig;

    public WeChatSignModel tokenByPost(String url, String bodyJson){
        return getToken("POST", HttpUrl.parse(url), bodyJson);
    }

    public WeChatSignModel tokenByGet(String url){
        return getToken("POST", HttpUrl.parse(url), "");
    }

    public WeChatSignModel getToken(String method, HttpUrl url, String body) {
        String nonceStr = IdUtils.simpleUUID();
        long timestamp = System.currentTimeMillis() / 1000;
        String message = buildMessage(method, url, timestamp, nonceStr, body);
        String signature = null;
        try {
            signature = sign(message.getBytes(StandardCharsets.UTF_8));
        } catch (NoSuchAlgorithmException | IOException | InvalidKeyException | SignatureException e) {
            throw new RuntimeException(e);
        }
        return WeChatSignModel.builder()
                .sign(
                        wechatPayConfig.getSchema()+" mchid=\""+wechatPayConfig.getMerchantId()+"\","
                                + "nonce_str=\""+nonceStr+"\","
                                + "timestamp=\""+timestamp+"\","
                                + "serial_no=\""+wechatPayConfig.getSerialNo()+"\","
                                + "signature=\""+signature+"\""
                )
                .timestamp(timestamp)
                .nonceStr(nonceStr)
                .build();
    }

    private String sign(byte[] message) throws NoSuchAlgorithmException, IOException, InvalidKeyException, SignatureException {
        Signature sign = Signature.getInstance(ALGORITHM_TYPE);
        sign.initSign(wechatPayConfig.getPrivateKey());
        sign.update(message);

        return Base64.getEncoder().encodeToString(sign.sign());
    }

    private String buildMessage(String method, HttpUrl url, long timestamp, String nonceStr, String body) {
        String canonicalUrl = url.encodedPath();
        if (url.encodedQuery() != null) {
            canonicalUrl += "?" + url.encodedQuery();
        }


        return method + "\n"
                + canonicalUrl + "\n"
                + timestamp + "\n"
                + nonceStr + "\n"
                + body + "\n";
    }

}
